
package com.embeddedartists.romfs;

/*
 * mkromfs.java
 *    Program to create a ROM File System image based on the contents
 *    of a directory tree.
 */

import java.io.*;
 

/*-----------------------------------------------------------------------
 *----------------------------------------------------------------------*/
public class Mkfs
{
  public static void main(String args[]) throws IOException
  {
    String              source = ".";
    OutputStream        dest = System.out;
    int                 byteOrder = 3;
    Image               bin;
    String              prefix = "";
    boolean             outputC = false;

    for (int i = 0;
             i < args.length;
             i++)
    {
      if (args[i].equals("-B"))
        byteOrder = Image.BIG_ENDIAN;

      else if (args[i].equals("-l"))
        byteOrder = Image.LITTLE_ENDIAN;

      else if (args[i].equals("-c"))
      {
        outputC = true;
        prefix = args[++i];
      }

      else if (args[i].equals("-o"))
        dest = new FileOutputStream(args[++i]);

      else if (args[i].equals("-d"))
        source = args[++i];

      else
        usage();
    }

    if (byteOrder != Image.BIG_ENDIAN && byteOrder != Image.LITTLE_ENDIAN)
      usage();

    bin = new Image(new Tree(source), byteOrder);

    if (outputC == true)
      bin.writeC(prefix, dest);
    else
      bin.writeB(dest);
  }

  private static void usage()
  {
    System.err.println(
        "Description:\n"
      + "  Create a File System Image from the contents of a local\n"
      + "  directory tree\n"
      + "\n"
      + "Synopsis:\n"
      + "\n"
      + "  mkfs -(B|l) [-c prefix] [-o filename] [-d directory]\n"
      + "\n"
      + "Options:\n"
      + "  -B            - Generate output in BIG ENDIAN byte order.\n"
      + "  -l            - Generate output in little endian byte order.\n"
      + "  -c prefix     - Write output as a compilable C array.\n"
      + "                  <prefix> is everything preceding the array data.\n"
      + "  -o filename   - Output file name (default is to use standard\n"
      + "                  output).\n"
      + "  -d directory  - Name of the input directory to read. Default is\n"
      + "                  to use the current working directory.\n"
      + "  -h            - Print this help message.\n"
      + "\n");
    System.exit(1);
  }
}
