/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __APPTABLE_H
#define __APPTABLE_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "application.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define ENABLE_GAMES
#define ENABLE_SLIDESHOW
#define ENABLE_GLOBEANIMATION
#define ENABLE_SPINNINGCUBE
#define ENABLE_JAWBREAKER
#define ENABLE_RANDOM_CIRCLES
#define ENABLE_MANDELBROT
#define ENABLE_BOUNCINGBALL


#define APP_TABLE_SIZE (sizeof(appTable) / sizeof(AppTableEntry_t))


/******************************************************************************
 * Forward Declarations of Initialization Functions
 *****************************************************************************/

// Mandatory Applications
AppCallback_t* menuAppInit(void);
AppCallback_t* calibAppInit(void);

#ifdef ENABLE_GAMES
AppCallback_t* fallingBlockGameAppInit(void);
#endif

#ifdef ENABLE_SLIDESHOW
AppCallback_t* slideshowAppInit(void);
#endif

#ifdef ENABLE_GLOBEANIMATION
AppCallback_t* globeAppInit(void);
#endif

#ifdef ENABLE_SPINNINGCUBE
AppCallback_t* cubeAppInit(void);
#endif

#ifdef ENABLE_JAWBREAKER
AppCallback_t* jawbreakerAppInit(void);
#endif

#ifdef ENABLE_RANDOM_CIRCLES
AppCallback_t* randomCirclesAppInit(void);
#endif

#ifdef ENABLE_MANDELBROT
AppCallback_t* mandelbrotAppInit(void);
#endif

#ifdef ENABLE_BOUNCINGBALL
AppCallback_t* bouncingballAppInit(void);
#endif

AppCallback_t* skeletonAppInit(void);


/******************************************************************************
 * Global Variables
 *****************************************************************************/

static const AppTableEntry_t
appTable[] =
{
  {APP_PRIO_1, &menuAppInit,             "MenuApp", TRUE},
  {APP_PRIO_2, &calibAppInit,            "Touch Screen Calibration", FALSE},
#ifdef ENABLE_GAMES
  {APP_PRIO_2, &fallingBlockGameAppInit, "Falling Block Game", FALSE},
#endif
#ifdef ENABLE_SLIDESHOW
  {APP_PRIO_2, &slideshowAppInit,        "Slideshow from SD Card", FALSE},
#endif
#ifdef ENABLE_GLOBEANIMATION
  {APP_PRIO_2, &globeAppInit,            "Globe Animation", FALSE},
#endif
#ifdef ENABLE_SPINNINGCUBE
  {APP_PRIO_2, &cubeAppInit,             "Cube Animation", FALSE},
#endif
#ifdef ENABLE_JAWBREAKER
  {APP_PRIO_2, &jawbreakerAppInit,       "Jawbreaker Game", FALSE},
#endif
#ifdef ENABLE_RANDOM_CIRCLES
  {APP_PRIO_2, &randomCirclesAppInit,    "Random Circles", FALSE},
#endif
#ifdef ENABLE_MANDELBROT
  {APP_PRIO_2, &mandelbrotAppInit,       "Mandelbrot Rendering", FALSE},
#endif
#ifdef ENABLE_BOUNCINGBALL
  {APP_PRIO_2, &bouncingballAppInit,     "Bouncing Ball", FALSE},
#endif
  {APP_PRIO_2, &skeletonAppInit,         "Not Used", FALSE},
  {APP_PRIO_2, &skeletonAppInit,         "Not Used", FALSE},
  {APP_PRIO_2, &skeletonAppInit,         "Not Used", FALSE},
};

#endif /* end __APPTABLE_H */
