/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __PROGRESS_H
#define __PROGRESS_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Creates a progress window (a white full screen window) and loads the
 *    necessary images but does not show the initial progress bar.
 *
 *    Note that a call to progress_set(0) should be made immediately after
 *    progress_init() to display the progress bar.
 *
 * Params:
 *    None
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void progress_init(void);


/*****************************************************************************
 *
 * Description:
 *    Sets the current progress in percent.
 *
 * Params:
 *    [in] percent   - The progress so far (range 0-100 inclusive)
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void progress_set(uint8_t percent);


/*****************************************************************************
 *
 * Description:
 *    Destroys the progress window and unloads the images.
 *
 * Params:
 *    None
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void progress_destroy(void);

#endif /* end __PROGRESS_H */

