/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __SWIM_EXTENSIONS_H
#define __SWIM_EXTENSIONS_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_swim.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Draws a circle in the virtual window. Reason for function is that the
 *    SWIM library does not support drawing of circles.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] cx      - Virtual X position of the circle's center
 *    [in] cy      - Virtual Y position of the circle's center
 *    [in] radius  - The radius of the circle
 *    [in] filled  - TRUE if the circle should be filled with the PEN color
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_put_circle(SWIM_WINDOW_T *win, int32_t cx, int32_t cy, int32_t radius, Bool Filled);

/******************************************************************************
 *
 * Description:
 *    Puts an raw image in a window unscaled, clips off edges. The image
 *    is drawn with it's top left corner at the specified position.
 *    The reason for this function is that the swim_put_image function in the
 *    SWIM library only puts the image at 0,0.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] image   - The image
 *    [in] xpos    - Virtual X position to put the top left corner of the image
 *    [in] ypos    - Virtual Y position to put the top left corner of the image
 *    [in] xsize   - The width of the image
 *    [in] ysize   - The height of the image
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_put_image_at_position(SWIM_WINDOW_T *win, const COLOR_T *image,
    int32_t xpos, int32_t ypos, int32_t xsize, int32_t ysize);

/******************************************************************************
 *
 * Description:
 *    Draws the text horizontally centered in the virtual window.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] y       - Virtual Y position to put the text
 *    [in] pText   - The text to draw
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_put_text_centered(SWIM_WINDOW_T *win, int32_t y, char *pText);

/******************************************************************************
 *
 * Description:
 *    Draws a filled rectangle.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] x1      - Virtual X position of the rectangle's top left corner
 *    [in] y1      - Virtual Y position of the rectangle's top left corner
 *    [in] x2      - Virtual X position of the rectangle's bottom right corner
 *    [in] y2      - Virtual Y position of the rectangle's bottom right corner
 *    [in] color   - The color to use
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_fill_rect(SWIM_WINDOW_T *win, int x1, int y1, int x2, int y2, COLOR_T color);

/******************************************************************************
 *
 * Description:
 *    Draws a filled circle.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] x1      - Virtual X position of the circle's center
 *    [in] y1      - Virtual Y position of the circle's center
 *    [in] radius  - The radius of the circle
 *    [in] color   - The color to use
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_fill_circle(SWIM_WINDOW_T *win, int x, int y, int radius, COLOR_T color);

/******************************************************************************
 *
 * Description:
 *    Draws a line with the specified color.
 *
 * Params:
 *    [in] win     - The window identifier
 *    [in] x1      - Virtual X position of the line's starting point
 *    [in] y1      - Virtual Y position of the line's starting point
 *    [in] x2      - Virtual X position of the line's end point
 *    [in] y2      - Virtual Y position of the line's end point
 *    [in] color   - The color to use
 *
 * Returns:
 *    None
 *
 *****************************************************************************/
void swimex_put_colored_line(SWIM_WINDOW_T *win, int x1, int y1, int x2, int y2, COLOR_T color);

#endif /* end __SWIM_EXTENSIONS_H */

