/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __JAW_TYPES_H
#define __JAW_TYPES_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define JAW_NUM_COLUMNS  10
#define JAW_NUM_ROWS     11
#define JAW_NUM_CELLS   108

#define JAW_NUM_TYPES     5

typedef enum
{
	CELL_STATE_EMPTY,
	CELL_STATE_SELECTED,
	CELL_STATE_UNSELECTED
} tCellState;

typedef struct
{
	int row;
	int col;
	int type;
	tCellState state;
	int lastRenderedState;
} tCell;

typedef tCell* tArrayListItem;

typedef struct
{
	tArrayListItem list[JAW_NUM_CELLS];
	int numUsed;
} tArrayList;


typedef struct
{
	tCell board[JAW_NUM_ROWS][JAW_NUM_COLUMNS];
	tArrayList selected;
	uint32_t totalScore;
} tBoard;

typedef enum
{
	E_OK,
	E_INVALID_POINTER,
	E_FAILED_TO_ALLOCATE,
	E_USER_SELECTED_TO_EXIT,
	E_INVALID_IMAGE,
	E_INVALID_FILE_FORMAT,
	E_FILE_PROBLEMS,
	E_LCD_CREATE_FAILED,
	E_INVALID_STATE
} tJawError;

/******************************************************************************
 * Public variables
 *****************************************************************************/


/******************************************************************************
 * Public functions
 *****************************************************************************/

tJawError jaw_ArrayList_Create(tArrayList** ppArrayList);
tJawError jaw_ArrayList_Destroy(tArrayList** ppArrayList);
tJawError jaw_ArrayList_Add(tArrayList* pArrayList, tCell* pCell);
tJawError jaw_ArrayList_AddIfUnique(tArrayList* pArrayList, tCell* pCell);
Bool jaw_ArrayList_Contains(tArrayList* pArrayList, tCell* pCell);

#endif /* end __JAW_TYPES_H */

