/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "application.h"
#include "jaw_game.h"
#include "log.h"
#include "tsc2046_touch.h"
#include "lpc177x_8x_timer.h"
#include "pca9532.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void
start(void)
{
  int x, y, z;
  int x2, y2, z2;
  Bool released = TRUE;
  tJawError result = E_OK;

  jaw_StartGame();

  x = y = z = 0;
  while (result == E_OK) {
    TIM_Waitms(100);
    touch_xyz(&x2, &y2, &z2);
    if (z2 == 0 && z != 0) {
      // ignore x and y coordinates as they are wrong when z becomes 0
      z = 0;
      released = TRUE;

      result = jaw_CursorAt(x, y, z, released);
    } else if ((x != x2) || (y != y2) || (z == 0 && z2 != 0)) {
      x = x2;
      y = y2;
      z = z2;
      if (released && z > 0) {
        released = FALSE;
      }

      result = jaw_CursorAt(x, y, z, released);
    } else if (pca9532_getLedState(FALSE) & KEY1) {
      /* Check SW2 button which causes Exit */
      result = E_USER_SELECTED_TO_EXIT;
    }

    switch(result)
    {
    case E_OK:
      break;
    case E_USER_SELECTED_TO_EXIT:
      log_i("jawbreakerApp: Use selected (inside game) to exit\r\n");
      break;
    default:
      log_e("jawbreakerApp: Error %u occurred. Terminating game\r\n", result);
    }
  }
}

static void
stop(void)
{
  log_i("jawbreakerApp: stop\r\n");

  jaw_EndGame();
}

/******************************************************************************
 * Implementation of public functions
 *****************************************************************************/

static AppCallback_t jawbreakerCallback = {
    &start,
    &stop,
    NULL,
    0,
    FALSE,
};

AppCallback_t* jawbreakerAppInit(void)
{
  return &jawbreakerCallback;
}
