/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "lpc_swim.h"
#include "pca9532.h"
#include "application.h"
#include "any_display.h"
#include "swim_extensions.h"
#include "log.h"
#include "random.h"

/******************************************************************************
 * Defines, macros, and typedefs
 *****************************************************************************/

/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

static SWIM_WINDOW_T MainWin;
static Bool started = FALSE;
static BackBuffer_t myLcdHnd1 = BACK_BUFFER_INVALID;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void runApp(void)
{
  int32_t WinWidth, WinHeight;
  int32_t RADIUS;

  if (!anydisplay_createBackBuffers(&myLcdHnd1, NULL, NULL))
  {
    log_e("Failed to allocate back buffers\r\n");
    return;
  }

  memset((void*)myLcdHnd1, 0, GlobalParameters.display.buffersize);

  random_init(msClock);

  /* Create a SWIM window */
  swim_window_open(&MainWin, GlobalParameters.display.width,
      GlobalParameters.display.height, (COLOR_T*)myLcdHnd1, 0, 0,
      (GlobalParameters.display.width - 1),
      (GlobalParameters.display.height - 1), 1, WHITE, BLACK, BLACK);
  anydisplay_activateBuffer(myLcdHnd1);

  WinWidth = MainWin.xpsize;
  WinHeight = MainWin.ypsize;
  started = TRUE;
  while (started) {
    RADIUS = random() % 50 + 15;
    swim_set_pen_color(&MainWin, (COLOR_T)(random() % NUM_COLORS));
    swimex_put_circle(&MainWin, random() % (WinWidth - RADIUS * 2) + RADIUS,
        random() % (WinHeight - RADIUS * 2) + RADIUS, RADIUS, TRUE);
  }
}

static void cleanup(void)
{
  started = FALSE;
  swim_window_close(&MainWin);
  anydisplay_freeBackBuffers(&myLcdHnd1, NULL, NULL);
}

static void timeout(void)
{
  if (started)
  {
    /* Check SW2 button which causes Exit */
    if (pca9532_getLedState(FALSE) & KEY1) {
      // end application
      started = FALSE;
    }
  }
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/

static AppCallback_t circlesCallback = {
    &runApp,
    &cleanup,
    &timeout,
    100,
    FALSE,
};

AppCallback_t* randomCirclesAppInit(void)
{
  return &circlesCallback;
}

