/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"
#include "lpc_swim.h"
#include "lpc_swim_font.h"
#include "application.h"
#include "pca9532.h"
#include "globals.h"
#include "log.h"
#include "swim_extensions.h"
#include "lpc177x_8x_timer.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

static SWIM_WINDOW_T BaseWindow;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void start(void)
{
  // Use the default frame buffer and use the entire screen
  swim_window_open(&BaseWindow,
      GlobalParameters.display.width,
      GlobalParameters.display.height,
      (COLOR_T*)GlobalParameters.display.framebuffer, 0, 0,
      GlobalParameters.display.width-1,
      GlobalParameters.display.height-1,
      1, BLACK, WHITE, WHITE);

  // Create a smaller window without border to display the text in
  swim_window_open(&BaseWindow,
      GlobalParameters.display.width,
      GlobalParameters.display.height,
      (COLOR_T*)GlobalParameters.display.framebuffer, 20, 20,
      GlobalParameters.display.width-20-1,
      GlobalParameters.display.height-20-1,
      0, BLACK, WHITE, WHITE);

  // Display the message
  swim_put_ltext(&BaseWindow, "The menu item you just clicked on does not exist.\n\n"
      "You can write your own App and add it to the menu system.");
  swimex_put_text_centered(&BaseWindow, BaseWindow.xpvmax - 20, "Press SW2 button to exit.");

  // Wait for user to click the button
  while (TRUE)
  {
    /* Check SW2 button which causes Exit */
    if (pca9532_getLedState(FALSE) & KEY1) {
      break;
    }

    /* Enough to test 10 times per second */
    TIM_Waitms(100);
  }
}

static void cleanup(void)
{
  swim_window_close(&BaseWindow);
}

static void timeout(void)
{
  log_i("Timeout in SkeletonApp\r\n");
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/

static AppCallback_t skeletonCallback = {
    &start,
    &cleanup,
    timeout,
    1000,      // Timeout every second
    FALSE,
};

AppCallback_t* skeletonAppInit(void)
{
  return &skeletonCallback;
}

