/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __SLIDESHOW_H
#define __SLIDESHOW_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"
#include "globals.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define SLIDE_WINDOW_WIDTH      GlobalParameters.display.width
#define SLIDE_WINDOW_HEIGHT     GlobalParameters.display.height
#define SLIDE_WINDOW_BPP        16
#define SLIDE_WINDOW_NUM_PIXELS (SLIDE_WINDOW_WIDTH * SLIDE_WINDOW_HEIGHT)
#define SLIDE_WINDOW_NUM_BYTES  (SLIDE_WINDOW_NUM_PIXELS * sizeof(uint16_t))

typedef enum
{
  SLIDE_OK,
  SLIDE_ERR_NO_SUCH_SCRIPT,
  SLIDE_ERR_INVALID_SCRIPT,
  SLIDE_ERR_RUNTIME,
  SLIDE_ERR_MALLOC,
  SLIDE_ERR_USER_ABORT,
  SLIDE_ERR_SCRIPT_END
} tSlideError;

/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Public functions
 *****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Run a slide show based on the specified script. If the script has an
 *    infinite loop it will continue to loop until *pAbort evaluates to TRUE.
 *
 * Params:
 *    [in] pScriptFile - Path to file with script to run.
 *    [in] pAbort      - Variable to poll for the abort condition
 *
 * Returns:
 *    SLIDE_INVALID_SCRIPT  if the script file couldn't be parsed
 *    SLIDE_RUNTIME_ERROR   if the execution of the script failed
 *    SLIDE_USER_ABORT      if the *pAbort is set to TRUE by the user
 *    SLIDE_SCRIPT_END      if the end of the script was reached
 *
 ****************************************************************************/
tSlideError slideRunSlideshow(void);

tSlideError slideRender(int32_t myLcdHnd, uint32_t tick);
tSlideError slideInit(char* pScriptFile, Bool* pAbort, Bool* pShouldRender, uint32_t* pDelayFactor);
void slideCleanup(void);

#endif /* end __SLIDESHOW_H */

