/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
#ifndef __TRANSITIONS_H
#define __TRANSITIONS_H

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc_types.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

typedef enum
{
  TRANSITION_NONE,
  TRANSITION_LEFT_RIGHT,
  TRANSITION_TOP_DOWN,
  TRANSITION_BLINDS,
  TRANSITION_EXPLODE_V, // For high/vertical placement of screen
  TRANSITION_EXPLODE_H, // For wide/horizontal placement of screen
  CLEAR_SCREEN  //must be last in enum as it is used when counting
} TTransition;

typedef uint16_t* TImage;

typedef struct
{
  char* pTransitionString;
  TTransition Transition;
  uint16_t NumRenderCallsNeeded;
  uint16_t (*showImage)(int32_t myLcdHnd, TImage CurrentImage, TImage NewImage, uint16_t callNo);
  uint16_t (*prepare)(TImage CurrentImage, TImage NewImage);
} TTransitionInfo;


#define TRANSITIONS_NUM_AVAILABLE   (CLEAR_SCREEN + 1)

/******************************************************************************
 * Public variables
 *****************************************************************************/

// The back buffer will be of twice normal screen size for some
// image magics
extern TImage ImageBackBuffer;

extern TTransitionInfo Transitions[TRANSITIONS_NUM_AVAILABLE+1];

/******************************************************************************
 * Public functions
 *****************************************************************************/

// sets next transition to be rendered
int prepareTransition(TTransition t, TImage CurrentImage, TImage NewImage);

// renders next part of current transition, returns true if last step was rendered
Bool renderTransition(int32_t myLcdHnd);

int initTransitions(void);
void cleanupTransitions(void);

#endif /* end __TRANSITIONS_H */

