/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc177x_8x_timer.h"
#include "lpc177x_8x_i2c.h"
#include "board.h"
#include "sdram.h"
#include "cfgmem.h"
#include "lpc_heap.h"
#include "any_display.h"
#include "log.h"
#include "application.h"
#include "tsc2046_touch.h"
#include "globals.h"
#include "rom_fs.h"
#include "diskio.h"


/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define I2C_PORT (I2C_0)

/******************************************************************************
 * Global variables
 *****************************************************************************/

/* Used by log.h */
uint8_t g_dbbuf[512];

/* Used by all apps to gain access to shared data */
GlobalParameters_t GlobalParameters;

/* The ROM file system */
extern const unsigned char file_fsimg[];

/******************************************************************************
 * Local variables
 *****************************************************************************/


/******************************************************************************
 * Local Functions
 *****************************************************************************/

static void greeting(void)
{
  log_i("\r\n***************************************************\r\n");
  log_i("*                                                 *\r\n");
  log_i("* Demo program for LPC1778 OEM Board...           *\r\n");
  log_i("* (C) Embedded Artists 2001-2012                  *\r\n");
  log_i("*                                                 *\r\n");
  log_i("***************************************************\r\n");
}

volatile uint32_t msClock = 0;
void SysTick_Handler(void)
{
  msClock +=10; //Using 100Hz system tick
  disk_timerproc();
}

/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{
  TIM_TIMERCFG_Type timerCfg;
  int32_t framebuf = (int32_t)(MEM_FB_BASE);
  lcdb_result_t result;

  /* Initialize devices */

  /* initialize timer */
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIM0, TIM_TIMER_MODE, &timerCfg);

  /* Generate system tick at 100Hz */
  SysTick_Config(SystemCoreClock/100);

  console_init();
  greeting();
  i2c0_pinConfig();

  /* Initialize I2C */
  I2C_Init(I2C_PORT, 100000);
  I2C_Cmd(I2C_PORT, ENABLE);

  sdram_init();
  lpc_heap_init((void *)MEM_HEAP_BASE, MEM_HEAP_SIZE);
  fsInit((void*)file_fsimg);

  result = anydisplay_init(framebuf);
  if (result == LCDB_RESULT_OK) {
    anydisplay_getParameters(&GlobalParameters.display);
    touch_init();

    // Run calibration once
    appStartApp("Touch Screen Calibration");
  }

  appStartInitial();
  return 0;
}

