/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include <stdlib.h>

/******************************************************************************
 * Defines, macros, and typedefs
 *****************************************************************************/

/*
 * The poor standard implementation of rand() is not at all random.
 * It is cyclic so that the pattern will repeat itself after about
 * 20 "random" values.
 */
#define RANDOM_CYCLE   20

/******************************************************************************
 * Public variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/

/*
 * Count the number of times that random() has been called and when
 * it reaches the end of the cycle create a new seed.
 */
static int numRandomValues = 0;

/******************************************************************************
 * Local functions
 *****************************************************************************/

/******************************************************************************
 * Implementation of public functions
 *****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Sets the seed to use for the random() function. This is equivalent to
 *    the srand() function in the standard library.
 *
 * Params:
 *    [in] seed     - the seed to initialize the random number sequence with
 *
 * Return:
 *    None
 *
 ****************************************************************************/
void random_init(int seed)
{
  srand(seed);
  numRandomValues = 0;
}

/*****************************************************************************
 *
 * Description:
 *    A substitute for the standard function rand() as it is poorly
 *    implemented and cyclic. This function should be replaced in the
 *    future with a better random number generator, but for now this is
 *    good enough.
 *
 * Params:
 *    None
 *
 * Return:
 *    The pseudo random number
 *
 ****************************************************************************/
int random(void)
{
  if (++numRandomValues == RANDOM_CYCLE)
  {
    random_init(rand());
  }
  return rand();
}
