/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/


#include <string.h>
#include <stdio.h>
#include "lpc_types.h"
#include "lpc177x_8x_pinsel.h"
#include "lpc177x_8x_uart.h"
#include "lpc177x_8x_ssp.h"
#include "lpc177x_8x_timer.h"

#include "sdram.h"
#include "truly_display.h"
#include "lcd_driver.h"
#include "lpc_swim.h"

#include "tsc2046_touch.h"

#include "board.h"

void calibrateStart(SWIM_WINDOW_T *win);

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define UART_PORT (LPC_UART0)


/******************************************************************************
 * Local Functions
 *****************************************************************************/



/******************************************************************************
 * Main method
 *****************************************************************************/

int main (void)
{

  SWIM_WINDOW_T win1;
  COLOR_T *fblog;

  TIM_TIMERCFG_Type timerCfg;

  int32_t x = 0;
  int32_t y = 0; 
  int32_t z = 0;

  int32_t framebuf = (int32_t)(SDRAM_BASE+0x10000);

  // initialize timer
  TIM_ConfigStructInit(TIM_TIMER_MODE, &timerCfg);
  TIM_Init(LPC_TIM0, TIM_TIMER_MODE, &timerCfg);

  console_init();
  
  sdram_init();

  memset((void*)(framebuf), 0x0, 240*320*2);
  /*dev_lcd = */truly_lcd_open(framebuf);

  touch_init();  

  fblog = (COLOR_T*)framebuf;

  /* Create a SWIM window */
  swim_window_open(&win1, DISPLAY_WIDTH, DISPLAY_HEIGHT, fblog, 0, 0, 
    DISPLAY_WIDTH-1, DISPLAY_HEIGHT-1, 1, WHITE, BLACK, BLACK);

  calibrateStart(&win1);
  swim_clear_screen(&win1, BLACK);

  while(1) {
    touch_xyz(&x, &y, &z);

    if (z > 0) {
      swim_put_pixel(&win1, x, y);
    }

    TIM_Waitms(2);
  }


  //truly_lcd_close(dev_lcd);
}


